@@ -11,7 +11,7 @@ from muffin.plugins import BasePlugin
 from peewee_migrate import Router
 
 
-__version__ = "1.7.11"
+__version__ = "1.7.14"
 __project__ = "muffin-peewee"
 __author__ = "Kirill Klenov <horneds@gmail.com>"
 __license__ = "MIT"
@@ -107,8 +107,10 @@ class Plugin(BasePlugin):
                 self.router.logger.info('Migrations are undone:')
                 self.router.logger.info('\n'.join(self.router.diff))
 
-        if self.cfg.manage_connections and self.is_async:
-            app.middleware(self.__middleware__)
+        if self.cfg.manage_connections:
+            db = self.database
+            md = self.is_async and async_middleware(db) or sync_middleware(db)
+            app.middleware(md, insert_first=True)
 
     def __getattr__(self, name: str) -> t.Any:
         """Proxy attrs to self database."""
@@ -251,3 +253,47 @@ class Choices:
         result = Choices(self._map.copy())
         memo[id(self)] = result
         return result
+
+
+def async_middleware(database) -> t.Callable[
+        [t.Callable, muffin.Request, Receive, Send], t.Awaitable]:
+    """Create a middleware for async databases."""
+
+    async def md(handler: t.Callable, request: muffin.Request, receive: Receive, send: Send):
+        await database.connect_async(reuse_if_open=True)
+
+        try:
+            response = await handler(request, receive, send)
+            database.commit()
+            return response
+
+        except pw.DatabaseError:
+            database.rollback()
+            raise
+
+        finally:
+            await database.close_async()
+
+    return md
+
+
+def sync_middleware(database) -> t.Callable[
+        [t.Callable, muffin.Request, Receive, Send], t.Awaitable]:
+    """Create a middleware for sync databases."""
+
+    async def md(handler: t.Callable, request: muffin.Request, receive: Receive, send: Send):
+        database.connect(reuse_if_open=True)
+
+        try:
+            response = await handler(request, receive, send)
+            database.commit()
+            return response
+
+        except pw.DatabaseError:
+            database.rollback()
+            raise
+
+        finally:
+            database.close()
+
+    return md
